@@ -1,4 +1,4 @@
-/*  Copyright (C) 2013 Ben Asselstine
+/*  Copyright (C) 2013, 2014 Ben Asselstine
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -8,7 +8,7 @@
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU Library General Public License for more details.
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -25,12 +25,13 @@
 #include "read-file.h"
 #include "error.h"
 #include "util.h"
+#include "url-downloader.h"
 
 static struct argp_option argp_options[] = 
 {
     {"2-clause", '2', NULL, 0, N_("show the 2-clause bsd license")},
     {"3-clause", '3', NULL, 0, N_("show the 3-clause bsd license")},
-    {"list-licenses", 'l', NULL, OPTION_HIDDEN, N_("show licenses and exit")},
+    {"list-license-notices", 'l', NULL, OPTION_HIDDEN, N_("show licenses and exit")},
     {0}
 };
 
@@ -50,9 +51,12 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
     case 'l':
         {
-          char **license = bsd.licenses;
-          while (*license)
-            fprintf (stdout, "%s\n", *license++);
+          int i = 0;
+          while (bsd.notices[i].keyword != NULL)
+            {
+              fprintf (stdout, "%s\n", bsd.notices[i].keyword);
+              i++;
+            }
           exit (0);
         }
       break;
@@ -93,35 +97,9 @@ show_lu_bsd(struct lu_state_t *state, struct lu_bsd_options_t *options)
     url = strdup ("http://directory.fsf.org/wiki?title=License:FreeBSD");
   else 
     url = strdup ("http://directory.fsf.org/wiki/License:BSD_3Clause");
-  int err = 0;
-  char tmp[sizeof(PACKAGE) + 13];
-  snprintf (tmp, sizeof tmp, "/tmp/%s.XXXXXX", PACKAGE);
-  int fd = mkstemp(tmp);
-  close (fd);
-  FILE *fileptr = fopen (tmp, "wb");
-  curl_easy_setopt (state->curl, CURLOPT_HTTPGET, 1);
-  curl_easy_setopt (state->curl, CURLOPT_URL, url);
-  curl_easy_setopt (state->curl, CURLOPT_WRITEDATA, fileptr);
-  curl_easy_perform(state->curl);
-  fflush (fileptr);
-  fsync (fileno (fileptr));
-  fclose (fileptr);
-  int response = 0;
-  curl_easy_getinfo (state->curl, CURLINFO_RESPONSE_CODE, &response);
-  if (response != 200)
-    {
-      remove (tmp);
-      error (0, 0, N_("got unexpected response code %d from %s"), response,
-             url);
-      err = 1;
-      return err;
-    }
+  char *data = NULL;
+  int err = download (state, url, &data);
   free (url);
-  fileptr = fopen (tmp, "r");
-  size_t data_len = 0;
-  char *data = fread_file (fileptr, &data_len);
-  fclose (fileptr);
-  remove (tmp);
 
   replace_html_entities (data);
 
@@ -137,7 +115,6 @@ show_lu_bsd(struct lu_state_t *state, struct lu_bsd_options_t *options)
     err = show_lines_after (state, data, 
                             "Redistribution and use in source and ", 
                             27, 0, NULL, NULL);
-  free (url);
   free (data);
   return err;
 }
@@ -154,13 +131,38 @@ struct lu_command_t bsd =
 {
   .name         = N_("bsd"),
   .doc          = BSD_DOC,
-  .flags        = IS_A_LICENSE | DO_NOT_SHOW_IN_HELP | SAVE_IN_HISTORY,
+  .flags        = IS_A_LICENSE | SAVE_IN_HISTORY | SHOW_IN_HELP,
   .argp         = &argp,
   .parser       = lu_bsd_parse_argp,
-  .licenses     = 
-    { 
-      "bsd3clause bsd",
-      "bsd2clause bsd --2-clause",
+  .latest_idx   = 0,
+  .mentions     =
+    {
+      "BSD license",
+      "Berkeley license",
+      "Berkeley Software Distribution",
+      "the BSD",
+      " BSD",
       NULL
+    },
+  .notices      =
+    {
+        {
+          .keyword   = "bsd3clause",
+          .cmd       = "bsd",
+          .words     = "Redistributions in binary form,(3)",
+          .recommend = 1
+        },
+        {
+          .keyword   = "bsd2clause",
+          .cmd       = "bsd --2-clause",
+          .words     = "Redistributions in binary form,(2)",
+          .recommend = 1
+        },
+        {
+          .keyword   = NULL,
+          .cmd       = NULL,
+          .words     = NULL,
+          .recommend = 0
+        },
     }
 };
