@@ -8,7 +8,7 @@
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU Library General Public License for more details.
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -30,6 +30,7 @@
 #include "error.h"
 #include "copy-file.h"
 #include "styles.h"
+#include "util.h"
 
 static struct argp_option argp_options[] = 
 {
@@ -65,7 +66,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
             argp_failure (state, 22, 0, 
                           N_("`%s' is not recommended!  "
                              "Use --force, "
-                             "or maybe try `%s+'"), arg, arg);
+                             "or maybe try `%s+' if the license is GNU related"), arg, arg);
           free (match);
         }
       else
@@ -84,8 +85,19 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case ARGP_KEY_FINI:
       if (opt->licenses == NULL)
         {
-          argp_failure (state, 0, 0, 
-                        N_("No license or comment style specified"));
+          char *style = lu_load_selected_comment_style ();
+          char *licenses = lu_load_selected_licenses ();
+          if (style || licenses)
+            {
+              char *msg = xasprintf (_("Selected: %s %s"), licenses, style);
+              argp_failure (state, 0, 0, msg);
+              free (msg);
+              free (style);
+              free (licenses);
+              argp_failure (state, 0, 0, "To change this try `choose LICENSE STYLE'");
+            }
+          else
+            argp_failure (state, 0, 0, "Try `choose LICENSE STYLE'");
           argp_state_help (state, stderr, ARGP_HELP_STD_ERR);
         }
       break;
@@ -107,11 +119,21 @@ help_filter (int key, const char *text, void *input)
           argz_add (&argz, &len, text);
           argz_add (&argz, &len, "");
         }
-      char *licenses = lu_list_of_license_keywords ();
-      char *l = xasprintf (N_("Supported Licenses: %s no-license"), licenses);
+      struct lu_license_t *licenses = lu_list_of_licenses (NULL);
+      int i = 0;
+      char *largz = NULL;
+      size_t largz_len = 0;
+      while (licenses[i].keyword != NULL)
+        {
+          argz_add (&largz, &largz_len, licenses[i].keyword);
+          i++;
+        }
+      argz_stringify (largz, largz_len, ' ');
+      free (licenses);
+      char *l = xasprintf (N_("Supported Licenses: %s no-license"), largz);
+      free (largz);
       argz_add (&argz, &len, l);
       free (l);
-      free (licenses);
       argz_add (&argz, &len, "");
       char *styles = lu_list_of_comment_styles();
       l = xasprintf ("Supported Comment Styles: %s no-style", styles);
@@ -145,25 +167,16 @@ lu_choose_parse_argp (struct lu_state_t *state, int argc, char **argv)
 }
 
 static char *
-get_command (char *argz, size_t len, char *license)
+get_command (struct lu_license_t *licenses, char *license)
 {
-  char *l = NULL;
-  while ((l = argz_next (argz, len, l)))
+  if (lu_is_license (license))
+    return lu_get_default_license_command (license);
+  int i = 0;
+  while (licenses[i].keyword != NULL)
     {
-      char *match = NULL;
-      if (sscanf (l, "%ms", &match) == 1)
-        {
-          if (strcmp (license, match) == 0)
-            {
-              free (match);
-              char *space = strchr (l, ' ');
-              space++;
-              while (*space == ' ' || *space == '\t')
-                space++;
-              return space;
-            }
-          free (match);
-        }
+      if (strcmp (licenses[i].keyword, license) == 0)
+        return licenses[i].cmd;
+      i++;
     }
   return NULL;
 }
@@ -171,40 +184,35 @@ get_command (char *argz, size_t len, char *license)
 static int 
 needs_force (char *arg)
 {
-  /* check to see if --jerkward is used in this license */
-  int need = 0;
-  char *cmds = lu_list_of_license_keyword_commands();
-  char *license_commands = NULL;
-  size_t len = 0;
-  argz_create_sep (cmds, '\n', &license_commands, &len);
-  free (cmds);
-  char *cmd = get_command(license_commands, len, arg);
-  if (cmd && strstr (cmd, "--jerkwad"))
-    need = 1;
-  free (license_commands);
-  return need;
+  int i = 0;
+  struct lu_license_t *licenses = lu_list_of_licenses (NULL);
+  while (licenses[i].keyword != NULL)
+    {
+      if (strcmp (licenses[i].keyword, arg) == 0)
+        {
+          int rec = licenses[i].recommend;
+          free (licenses);
+          return !rec;
+        }
+      i++;
+    }
+  free (licenses);
+  return 0;
 }
 
-
 static char *
-get_license_by_command (char *argz, size_t len, char *command)
+get_license_by_command (struct lu_license_t *licenses, char *command)
 {
-  char *l = NULL;
-  while ((l = argz_next (argz, len, l)))
+  if (lu_is_license (command))
+    return strdup (command);
+  int i = 0;
+  while (licenses[i].keyword != NULL)
     {
-      char *match = NULL;
-      char *license = NULL;
-      if (sscanf (l, "%ms %ms", &license, &match) == 2)
-        {
-          if (strcmp (command, match) == 0)
-            {
-              free (match);
-              return license;
-            }
-          free (match);
-          free (license);
-        }
+      if (strcmp (command, licenses[i].cmd) == 0)
+        return strdup (licenses[i].keyword);
+      i++;
     }
+
   return NULL;
 }
 
@@ -212,16 +220,17 @@ static int
 lookup (char *arg, char **match)
 {
   int valid = 1;
-  char *cmds = lu_list_of_license_keyword_commands();
-  char *license_commands = NULL;
-  size_t len = 0;
-  argz_create_sep (cmds, '\n', &license_commands, &len);
-  free (cmds);
+  struct lu_license_t *licenses = lu_list_of_licenses(NULL);
 
-  char *cmd = get_command (license_commands, len, arg);
+  if (lu_is_license (arg))
+    {
+      *match = lu_get_default_license (arg);
+      arg = *match;
+    }
+  char *cmd = get_command (licenses, arg);
   if (!cmd)
     {
-      char *license = get_license_by_command (license_commands, len, arg);
+      char *license = get_license_by_command (licenses, arg);
       if (!license && !lu_is_a_comment_style (arg))
         valid = 0;
       if (license)
@@ -243,29 +252,10 @@ lookup (char *arg, char **match)
       valid = 1;
     }
 
-  free (license_commands);
+  free (licenses);
   return valid;
 }
 
-static char *
-load_selected_comment_style ()
-{
-  char *data = NULL;
-  char *f = get_config_file ("selected-comment-style");
-  if (f)
-    {
-      FILE *fp = fopen (f, "r");
-      if (fp)
-        {
-          size_t data_len = 0;
-          data = fread_file (fp, &data_len);
-          fclose (fp);
-        }
-      free (f);
-    }
-  return data;
-}
-
 static void
 write_selected_comment_style (char *style)
 {
@@ -287,24 +277,6 @@ write_selected_comment_style (char *style)
     }
 }
 
-static char *
-load_selected_licenses ()
-{
-  char *data = NULL;
-  char *f = get_config_file ("selected-licenses");
-  if (f)
-    {
-      FILE *fp = fopen (f, "r");
-      if (fp)
-        {
-          size_t data_len = 0;
-          data = fread_file (fp, &data_len);
-          fclose (fp);
-        }
-      free (f);
-    }
-  return data;
-}
 
 static int
 write_selected_licenses (struct lu_state_t *state, struct lu_choose_options_t *options)
@@ -318,7 +290,7 @@ write_selected_licenses (struct lu_state_t *state, struct lu_choose_options_t *o
       remove (f);
       return err;
     }
-  char *licenses = options->licenses;
+  char *selected_licenses = options->licenses;
   char *f = get_config_file ("license-notice");
   if (f)
     {
@@ -329,39 +301,49 @@ write_selected_licenses (struct lu_state_t *state, struct lu_choose_options_t *o
       FILE *fp = fopen (tmp, "w");
       if (fp)
         {
-          char *cmds = lu_list_of_license_keyword_commands();
-          char *license_commands = NULL;
-          size_t license_len = 0;
-          argz_create_sep (cmds, '\n', &license_commands, &license_len);
-          free (cmds);
+          struct lu_license_t *licenses = lu_list_of_licenses (NULL);
+          int i = 0;
           char *argz = NULL;
           size_t argz_len = 0;
-          argz_create_sep (options->licenses, ' ', &argz, &argz_len);
-          char *l = NULL;
-          size_t count = argz_count (argz, argz_len);
-          while ((l = argz_next (argz, argz_len, l)))
+          argz_create_sep (selected_licenses, ' ', &argz, &argz_len);
+          int count = argz_count (argz, argz_len);
+          while (licenses[i].keyword != NULL)
             {
-              char *cmd = get_command (license_commands, license_len, l);
-              FILE *old_out = state->out;
-              state->out = fp;
-              err = lu_parse_command (state, cmd);
-              if (err)
+              int found = 0;
+              char *l = NULL;
+              while ((l = argz_next (argz, argz_len, l)))
                 {
+                  if (strcmp (l, licenses[i].keyword) == 0)
+                    {
+                      found = 1;
+                      break;
+                    }
+                }
+              //is this in selected licenses?
+              if (found)
+                {
+                  FILE *old_out = state->out;
+                  state->out = fp;
+                  err = lu_parse_command (state, licenses[i].keyword);
+                  if (err)
+                    {
+                      state->out = old_out;
+                      break;
+                    }
+                  if (count > 1)
+                    {
+                      fprintf (fp, "\n");
+                      fprintf (fp, "---\n");
+                      fprintf (fp, "\n");
+                    }
                   state->out = old_out;
-                  break;
                 }
-              if (count > 1)
-                {
-                  fprintf (fp, "\n");
-                  fprintf (fp, "---\n");
-                  fprintf (fp, "\n");
-                }
-              state->out = old_out;
+              i++;
               count--;
             }
+          free (licenses);
           free (argz);
           fclose (fp);
-          free (license_commands);
           //install it
           if (!err)
             {
@@ -380,7 +362,7 @@ write_selected_licenses (struct lu_state_t *state, struct lu_choose_options_t *o
           FILE *fp = fopen (f, "w");
           if (fp)
             {
-              fprintf (fp, "%s", licenses ? licenses: "");
+              fprintf (fp, "%s", selected_licenses ? selected_licenses: "");
               fclose (fp);
             }
           free (f);
@@ -393,8 +375,8 @@ int
 lu_choose (struct lu_state_t *state, struct lu_choose_options_t *options)
 {
   int err = 0;
-  char *old_style = load_selected_comment_style ();
-  char *old_license = load_selected_licenses ();
+  char *old_style = lu_load_selected_comment_style ();
+  char *old_license = lu_load_selected_licenses ();
   char *l = NULL;
   char *style = NULL;
   int no_license = 0;
@@ -475,7 +457,10 @@ lu_choose (struct lu_state_t *state, struct lu_choose_options_t *options)
   free (old_style);
   free (old_license);
   if (options->quiet == 0 && !err)
-    fprintf (stderr, "Selected.\n");
+    {
+      program_invocation_name = choose.name;
+      error (0, 0, "Selected.");
+    }
   return 0;
 }
 
@@ -483,7 +468,7 @@ struct lu_command_t choose =
 {
   .name         = N_("choose"),
   .doc          = CHOOSE_DOC,
-  .flags        = SHOW_IN_HELP | SAVE_IN_HISTORY,
+  .flags        = SHOW_IN_HELP | SAVE_IN_HISTORY | IS_MOST_FREQUENTLY_USED,
   .argp         = &argp,
   .parser       = lu_choose_parse_argp
 };
