@@ -1,4 +1,4 @@
-/*  Copyright (C) 2013 Ben Asselstine
+/*  Copyright (C) 2013, 2014 Ben Asselstine
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -8,7 +8,7 @@
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU Library General Public License for more details.
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -25,10 +25,11 @@
 #include "read-file.h"
 #include "error.h"
 #include "util.h"
+#include "url-downloader.h"
 
 static struct argp_option argp_options[] = 
 {
-    {"list-licenses", 'l', NULL, OPTION_HIDDEN, N_("show licenses and exit")},
+    {"list-license-notices", 'l', NULL, OPTION_HIDDEN, N_("show licenses and exit")},
     {0}
 };
 
@@ -39,9 +40,12 @@ parse_opt (int key, char *arg, struct argp_state *state)
     {
     case 'l':
         {
-          char **license = mit.licenses;
-          while (*license)
-            fprintf (stdout, "%s\n", *license++);
+          int i = 0;
+          while (mit.notices[i].keyword != NULL)
+            {
+              fprintf (stdout, "%s\n", mit.notices[i].keyword);
+              i++;
+            }
           exit (0);
         }
       break;
@@ -73,35 +77,9 @@ int
 show_lu_mit(struct lu_state_t *state, struct lu_mit_options_t *options)
 {
   char *url = strdup ("http://directory.fsf.org/wiki/License:X11");
-  int err = 0;
-  char tmp[sizeof(PACKAGE) + 13];
-  snprintf (tmp, sizeof tmp, "/tmp/%s.XXXXXX", PACKAGE);
-  int fd = mkstemp(tmp);
-  close (fd);
-  FILE *fileptr = fopen (tmp, "wb");
-  curl_easy_setopt (state->curl, CURLOPT_HTTPGET, 1);
-  curl_easy_setopt (state->curl, CURLOPT_URL, url);
-  curl_easy_setopt (state->curl, CURLOPT_WRITEDATA, fileptr);
-  curl_easy_perform(state->curl);
-  fflush (fileptr);
-  fsync (fileno (fileptr));
-  fclose (fileptr);
-  int response = 0;
-  curl_easy_getinfo (state->curl, CURLINFO_RESPONSE_CODE, &response);
-  if (response != 200)
-    {
-      remove (tmp);
-      error (0, 0, N_("got unexpected response code %d from %s"), response,
-             url);
-      err = 1;
-      return err;
-    }
+  char *data = NULL;
+  int err = download (state, url, &data);
   free (url);
-  fileptr = fopen (tmp, "r");
-  size_t data_len = 0;
-  char *data = fread_file (fileptr, &data_len);
-  fclose (fileptr);
-  remove (tmp);
 
   replace_html_entities (data);
   err = show_lines_after (state, data, "Permission is hereby granted", 18, 0, 
@@ -122,12 +100,30 @@ struct lu_command_t mit =
 {
   .name         = N_("mit"),
   .doc          = MIT_DOC,
-  .flags        = IS_A_LICENSE | DO_NOT_SHOW_IN_HELP | SAVE_IN_HISTORY,
+  .flags        = IS_A_LICENSE | SAVE_IN_HISTORY | SHOW_IN_HELP,
   .argp         = &argp,
   .parser       = lu_mit_parse_argp,
-  .licenses     = 
-    { 
-      "mit mit",
+  .latest_idx   = 0,
+  .mentions     =
+    {
+      "MIT license",
+      "the MIT",
+      " MIT",
       NULL
+    },
+  .notices      =
+    {
+        {
+          .keyword   = "mit",
+          .cmd       = "mit",
+          .words     = "furnished",
+          .recommend = 1
+        },
+        {
+          .keyword   = NULL,
+          .cmd       = NULL,
+          .words     = NULL,
+          .recommend = 0
+        },
     }
 };
